from dac import *
from time import sleep
from datetime import datetime
from influxdb_client import InfluxDBClient, client, domain, Point
import numpy as np

LOOP_TIME = 2

class Asservissement():

    def __init__(self) -> None:
        self.mydac = MCP4725(url='ftdi://ftdi:232h:FT1ZT0DW/1')

        self.inner_consigne_base = 19.5
        self.inner_consigne = self.inner_consigne_base

        self.inner_mesure = 0
        self.inner_erreur = 0
        self.inner_err_hist = [0.]*15 + [float('nan')]
        self.inner_erreur_integree = 0

        self.inner_Kp = 0.2
        self.inner_Ti = 800
        self.inner_Td = -120

        self.valve_default = 2.5
        self.cmd_to_valve = lambda x: 0.5 * max( 0, ((1/0.5)*self.valve_default)**(1/2.0) + x )**2.0
        self.time_since_start = 0

        self.outer_consigne = 23

        self.outer_mesure = 0
        self.outer_erreur = 0
        self.outer_erreur_integree = 0

        self.outer_Kp = 0.1
        self.outer_Ti = 600

        self.client_db = InfluxDBClient(url=f"http://127.0.0.1:8086/", 
                        token="NnB9OxRT1fHETIx3UH98CFwl_bUmQ9yxTcTSMGC0sGQkbwzmOUJF4HDZgJ4ddzjrNT-uUOmwcHET2Sj95MQpzg==", 
                        org="Strontium")
        self.query_api = self.client_db.query_api()

        self.db_write_api = self.client_db.write_api(write_options=client.write_api.SYNCHRONOUS)

    def db_write_point(self, p):
        p.time(datetime.utcnow(), write_precision=domain.write_precision.WritePrecision.MS)
        self.db_write_api.write(bucket="lab", record=p)


    def run(self) -> None:
        while(True):

            def get_temp(where):
                req_mesure = f'from(bucket: "lab") \
                    |> range(start: -5m, stop: now()) \
                    |> filter(fn: (r) => r["_measurement"] == "roomtemp") \
                    |> filter(fn: (r) => r["_field"] == "tempC") \
                    |> filter(fn: (r) => r["where"] == "{where}") \
                    |> last()'

                return self.query_api.query(req_mesure)[0].records[0].values["_value"]

            ### Boucle de régulation intérieure ###

            self.inner_mesure = get_temp("airsock")

            self.inner_erreur = self.inner_consigne - self.inner_mesure
            last_I = self.inner_erreur_integree
            self.inner_erreur_integree += LOOP_TIME * self.inner_erreur
            self.inner_err_hist.append(self.inner_erreur)
            self.inner_err_hist.pop(0)
            self.inner_err_deriv = -np.nanmean(np.diff(self.inner_err_hist)) / LOOP_TIME

            cmd = - ( self.inner_Kp * self.inner_erreur + 1/self.inner_Ti * self.inner_erreur_integree + self.inner_Td * self.inner_err_deriv )

            print(f'[Boucle intérieure]\n Erreur: {self.inner_erreur:.2f} \n Erreur intégrée: {self.inner_erreur_integree:.2f}\n Erreur dérivée: {self.inner_err_deriv:.3f}\n Kp: {self.inner_Kp:.2f}\n Commande: {cmd:.2f}')

            cmd = self.cmd_to_valve(cmd)

            r = self.time_since_start / (self.inner_Ti / 2.)
            if r < 1:
                cmd = r * cmd + (1-r) * self.valve_default
            self.time_since_start += LOOP_TIME

            # saturation de l'intégrateur et bornage de la sortie
            if cmd > 6:
                cmd = 6.
                self.inner_erreur_integree = last_I
            elif cmd < 0:
                cmd = 0.
                self.inner_erreur_integree = last_I

            ### Boucle de régulation extérieure ###

            self.outer_mesure = get_temp("sciencetable")

            self.outer_erreur = self.outer_consigne - self.outer_mesure
            last_I = self.outer_erreur_integree
            self.outer_erreur_integree += LOOP_TIME * self.outer_erreur

            cmd_inner = self.outer_Kp * self.outer_erreur + 1/self.outer_Ti * self.outer_erreur_integree
            self.inner_consigne = self.inner_consigne_base + cmd_inner

            # saturation de l'intégrateur et bornage de la sortie
            if self.inner_consigne > self.outer_consigne:
                self.inner_consigne = self.outer_consigne
                self.outer_erreur_integree = last_I
            if self.inner_consigne < 17:
                self.inner_consigne = 17.
                self.outer_erreur_integree = last_I

            print(f'[Boucle extérieure]\n Mesure: {self.outer_mesure:.2f} °C\n Commande: {self.inner_consigne:.2f}°C \n Erreur: {self.outer_erreur:.2f} \n Erreur intégrée: {self.outer_erreur_integree:.2f}')

            self.mydac.open()
            self.mydac.scale = (0, 10)
            self.mydac.voltage = 2.5
            self.mydac.close()

            self.db_write_point( Point("asservissement").field("vanne", float(cmd)) )
            self.db_write_point( Point("roomtemp").tag("where","consigne airsock").field("tempC", float(self.inner_consigne)) )

            sleep(LOOP_TIME)

if __name__ == "__main__":
    asser = Asservissement()
    asser.run()
